# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import logging
from importlib import import_module

from django.contrib.contenttypes.fields import GenericRelation
from django.core.cache import caches
from django.db import models
from django.db.models import DateTimeField
from django.utils.encoding import python_2_unicode_compatible
from django.utils.module_loading import import_string
from django.utils.text import slugify
from django.utils.translation import ugettext, ugettext_lazy as _
from first import first
from touchtechnology.admin.mixins import AdminUrlMixin
from touchtechnology.common.db.models import BooleanField, HTMLField, TemplatePathField
from touchtechnology.content.app_settings import (
    NODE_CACHE, PAGE_CONTENT_CLASSES, PAGE_TEMPLATE_BASE, PAGE_TEMPLATE_FOLDER, PAGE_TEMPLATE_REGEX,
)

logger = logging.getLogger(__name__)

PAGE_CONTENT_CLASS_CHOICES = zip(PAGE_CONTENT_CLASSES,
                                 PAGE_CONTENT_CLASSES)

PAGE_CONTENT_CLASS_DEFAULT = first(PAGE_CONTENT_CLASSES)

SITE_CACHE_KEY = '_site_cache'


class AdminUrlModel(AdminUrlMixin, models.Model):

    class Meta:
        abstract = True

    def _get_admin_namespace(self):
        return 'admin:content:%s' % self._meta.model_name

    def _get_url_args(self):
        return (self.id,)

    def __repr__(self):
        return '<{}: {}>'.format(self.__class__, self)


class Page(models.Model):

    # allows us to customise the template on a per page basis

    template = TemplatePathField(max_length=200, blank=True,
                                 template_base=PAGE_TEMPLATE_BASE,
                                 template_folder=PAGE_TEMPLATE_FOLDER,
                                 match=PAGE_TEMPLATE_REGEX,
                                 recursive=True,
                                 verbose_name=_("Template"))

    # SEO metadata fields

    keywords = models.CharField(
        max_length=255, blank=True, verbose_name=_("Keywords"),
        help_text=_("This should be a comma-separated list of terms that "
                    "indicate the content of this page - used to assist "
                    "Search Engines rank your page."))

    description = models.CharField(
        max_length=255, blank=True, verbose_name=_("Description"),
        help_text=_("Search Engines may use this when determining the "
                    "relevance of your page."))

    last_modified = DateTimeField(auto_now=True)

    def __repr__(self):
        return '<Page: #{}>'.format(self.pk)


class Content(models.Model):

    # FIXME and put the ``copy`` attribute on the ``PageContent`` model, we
    # aren't actually reusing the ``Content`` in any way, so simplify the
    # implementation to move the data back up and onto.

    # The M2M relation on ``Page`` will become a FK on ``PageContent``
    # instead.

    copy = HTMLField(blank=True)

    def __repr__(self):
        return '<Content: #%d>' % self.pk


class PageContent(models.Model):

    page = models.ForeignKey(
        Page, related_name='content', verbose_name=_("Page"),
        on_delete=models.PROTECT)
    label = models.SlugField(
        max_length=20, choices=PAGE_CONTENT_CLASS_CHOICES,
        default=PAGE_CONTENT_CLASS_DEFAULT, verbose_name=_("CSS class"))
    sequence = models.PositiveIntegerField(verbose_name=_("Sequence"))
    copy = HTMLField(blank=True, verbose_name=_("Copy"))

    last_modified = DateTimeField(auto_now=True)

    class Meta:
        ordering = ('sequence',)

    def __repr__(self):
        return '<PageContent: #%d, page=%d>' % (self.pk, self.page.pk)


@python_2_unicode_compatible
class Chunk(AdminUrlModel):

    slug = models.SlugField(verbose_name=_("Slug"))
    copy = HTMLField(blank=True, verbose_name=_("Page Copy"))

    def __str__(self):
        return self.slug


class NodeContent(models.Model):

    node = models.ForeignKey(
        'common.SitemapNode', related_name='contents', verbose_name=_("Node"),
        on_delete=models.PROTECT)
    copy = HTMLField(blank=True, verbose_name=_("Page Copy"))


@python_2_unicode_compatible
class Placeholder(models.Model):

    path = models.CharField(max_length=255, verbose_name=_("Module path"))
    namespace = models.CharField(
        max_length=255, db_index=True, verbose_name=_("Namespace"))

    nodes = GenericRelation(
        'common.SitemapNode', verbose_name=_("Nodes"))

    class Meta:
        ordering = ('path',)

    def __str__(self):
        try:
            cls = import_string(self.path)
        except (ImportError, ValueError):
            return ugettext('Not installed')
        return cls.verbose_name()

    def _install(self, parent=None):
        """
        Adding an internal method to allow us to rig up all the applications.
        """
        if not self.nodes.count():
            module_path, site = self.path.rsplit('.', 1)
            module = import_module(module_path)
            if hasattr(module, site):
                instance = getattr(module, site)
                self.nodes.create(
                    title=instance.name.title(),
                    slug=slugify(instance.name),
                    parent=parent,
                )

    @property
    def module(self):
        if not hasattr(self, '_module'):
            module_path = self.path.split('.')
            while module_path:
                try:
                    module = import_module('.'.join(module_path))
                    if hasattr(module, 'VERSION'):
                        self._module = module
                        break
                except ImportError:
                    module = None
                module_path.pop()
        return self._module

    def callable(self):
        try:
            return import_string(self.path)
        except ValueError:
            return None

    def site(self, node):
        cache_key = "node%d" % node.pk
        cls = import_string(self.path)
        site = cls(node=node, name=cache_key, **node.kwargs)
        return site

    def invalidate_cache(self):
        # Since this is forced cache invalidation, we should log it.
        logger.info("Forced invalidation of Application cache.")
        cache = caches[NODE_CACHE]
        return cache.clear()
    invalidate_cache.alters_data = True


@python_2_unicode_compatible
class Redirect(AdminUrlModel):

    source_url = models.CharField(
        max_length=250, verbose_name=_("Source URL"),
        help_text=_("The path that will trigger the redirection."))

    destination_url = models.CharField(
        max_length=500, verbose_name=_("Destination URL"),
        help_text=_("The URL or path that the browser will be sent to."))

    label = models.CharField(
        max_length=100, blank=True, verbose_name=_("Label"))

    active = BooleanField(default=True, verbose_name=_("Active"))
    permanent = BooleanField(default=False, verbose_name=_("Permanent"))

    class Meta:
        ordering = ('destination_url',)

    def __str__(self):
        if self.label:
            return self.label
        return '{src} → {dst}'.format(
            src=self.source_url, dst=self.destination_url)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('competition', '0009_remove_byeteam'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='matchvideo',
            options={'verbose_name': 'video'},
        ),
        migrations.RemoveField(
            model_name='matchvideo',
            name='approved_by',
        ),
        migrations.AlterField(
            model_name='matchvideo',
            name='url',
            field=models.URLField(max_length=255, verbose_name='URL'),
            preserve_default=True,
        ),
    ]



import logging
from decimal import Decimal, DivisionByZero, InvalidOperation

from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Count, Sum
from tournamentcontrol.competition.signals.decorators import (
    disable_for_loaddata,
)
from tournamentcontrol.competition.utils import SumDict

logger = logging.getLogger(__name__)

aggregate_kw = dict(
    played=Sum('played'),
    win=Sum('win'),
    loss=Sum('loss'),
    draw=Sum('draw'),
    bye=Sum('bye'),
    forfeit_for=Sum('forfeit_for'),
    forfeit_against=Sum('forfeit_against'),
    score_for=Sum('score_for'),
    score_against=Sum('score_against'),
    bonus_points=Sum('bonus_points'),
    points=Sum('points'),
)


@disable_for_loaddata
def changed_points_formula(sender, instance, *args, **kwargs):
    """
    When the ``points_formula`` is edited, we need to trigger a rebuild of all
    matches that have results and are related to the updated instance.
    """
    if 'points_formula' in instance.changed_fields:
        for m in instance.matches.filter(ladder_entries__isnull=False):
            m.save()


@disable_for_loaddata
def scale_ladder_entry(sender, instance, *args, **kwargs):
    """
    When there are pools of different sizes, it may be desirable to scale the
    number of points earned in the smaller group/s to allow comparison with
    larger groups.

    If enabled, this evaulation will be performed by applying the formula

        value / t * T

    where ``t`` is the number of teams in the small group, and ``T`` is the
    number of teams in the largest group.
    """
    if not instance.stage.scale_group_points:
        logger.debug('%r has not enabled point scaling, skip.',
                     instance.stage)
        return

    if instance.stage_group is None:
        logger.debug('%r is not part of a stage with pools, skip.',
                     instance)
        return

    # Precalculation to assist in group size calculations. Exclude matches
    # that are marked as a Bye as these are "unplayable" and are effectively
    # the difference between divisions.
    stages = instance.stage.pools.exclude(matches__is_bye=True)
    stages = stages.annotate(count=Count('matches'))

    # Determine the size of the group in question, and the largest group in
    # this Stage.
    t = instance.stage_group.teams.count()
    T = max(instance.stage.pools.annotate(
            count=Count('teams')).values_list('count', flat=True))

    if t == T:
        logger.debug('%r is one of the large pools, skip.',
                     instance.stage_group)
        return

    # Convert ``t`` and ``T`` to Decimal type
    t = Decimal(t)
    T = Decimal(T)

    # Determine the adjusted value for the points
    points = instance.points / t * T
    logger.debug('%s / %s * %s = %s', instance.points, t, T, points)

    # Update the points value after adjustment
    instance.difference = instance.difference / t * T
    instance.percentage = instance.percentage / t * T
    instance.points = instance.points / t * T


@disable_for_loaddata
def team_ladder_entry_aggregation(sender, instance, created=None,
                                  *args, **kwargs):
    """
    Function to be called following a LadderEntry being saved.

    The should recalculate the LadderSummary for a particular team in a
    particular division so that we don't have to do too many database hits and
    calculations for ladders.

    We can also sort a ladder for a division easily this way without need to
    calculate and then compare.
    """
    try:
        instance.team.ladder_summary.filter(
            stage=instance.match.stage).delete()
    except ObjectDoesNotExist as e:
        logger.debug(e)

    # if we are carrying points from the previous stage then we'll need to add
    # them here.

    base = SumDict()
    if instance.match and instance.match.stage_group:
        home_pks = instance.match.stage_group.matches.values_list(
            'home_team', flat=True)
        away_pks = instance.match.stage_group.matches.values_list(
            'away_team', flat=True)
        opponent_pks = set(home_pks).union(away_pks).difference(
            [instance.team_id])

        # when the Pool has carry_ladder set we want to only bring forward the
        # statistics from matches played with other teams in the group.
        if instance.match.stage_group.carry_ladder:
            logger.debug('Pool match, group statistics only.')
            base += instance.team.ladder_entries.filter(
                match__stage=instance.match.stage.comes_after,
                opponent__in=opponent_pks).aggregate(**aggregate_kw)

        # when the Stage has carry_ladder set but not the Pool we want to
        # bring forward all the teams statistics from the preceding stage.
        elif instance.match.stage.carry_ladder:
            logger.debug('Pool match, all stage statistics.')
            base += instance.team.ladder_entries.filter(
                match__stage=instance.match.stage.comes_after).aggregate(
                **aggregate_kw)

    elif instance.match.stage.carry_ladder:
        # when the preceeding stage had Pools, this stage does not have any
        # Pools, and carry_ladder is set, then we would bring forward the
        # statistics from matches played with other teams in the stage.
        if instance.match.stage.comes_after.pools.count():
            base += instance.team.ladder_entries.filter(
                match__stage=instance.match.stage.comes_after,
                opponent__in=instance.match.stage.teams).aggregate(
                **aggregate_kw)

        # when there are no Pools and the stage has carry_ladder set we bring
        # forward the ladder_summary instead.
        else:
            base += instance.team.ladder_summary.filter(
                stage=instance.match.stage.comes_after).aggregate(
                **aggregate_kw)

    aggregate = base + instance.team.ladder_entries.filter(
        match__stage=instance.match.stage).aggregate(**aggregate_kw)

    score_for = aggregate.get('score_for')
    score_against = aggregate.get('score_against')

    try:
        difference = score_for - score_against
    except TypeError:
        difference = None

    try:
        percentage = Decimal(score_for) / Decimal(score_against) * Decimal(100)
    except (DivisionByZero, InvalidOperation, TypeError):
        percentage = None

    aggregate.update({'difference': difference})
    aggregate.update({'percentage': percentage})

    instance.team.ladder_summary.create(
        stage=instance.match.stage,
        stage_group=instance.match.stage_group,
        **aggregate)



# coding=UTF-8
from __future__ import unicode_literals

from datetime import date, datetime, time

import pytz
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.test import TestCase
from django.test.utils import override_settings
from django.utils import timezone
from touchtechnology.common.forms.auth import permissionformset_factory
from touchtechnology.common.forms.fields import (
    EmailField, HTMLField, SelectDateField, SelectDateTimeField, SelectTimeField,
)
from touchtechnology.common.models import SitemapNode
from touchtechnology.common.tests import factories


class CustomFormField(TestCase):

    def test_email_field(self):
        self.assertFieldOutput(
            EmailField,
            {'a@a.com': 'a@a.com', 'B@B.COM': 'b@b.com'},
            {'aaa': ['Enter a valid email address.']})
        self.assertFieldOutput(
            EmailField,
            {'a@a.com': 'a@a.com', 'B@B.COM': 'B@B.COM'},
            {'aaa': ['Enter a valid email address.']},
            (),
            {'lowercase': False})

    def test_html_field(self):
        valid = {
            '<a href="http://www.example.com/">Example</a>':
                '<a href="http://www.example.com/">Example</a>',
            'Penn\u00a0& Teller': 'Penn&nbsp;& Teller',
            'sauté': 'saut&eacute;',
        }
        self.assertFieldOutput(HTMLField, valid, {})

    def test_select_date_field(self):
        valid = {
            ('25', '9', '2013'): date(2013, 9, 25),
        }
        invalid = {
            ('30', '2', '2013'): ['Please enter a valid date.'],
        }
        self.assertFieldOutput(SelectDateField, valid, invalid,
                               empty_value=None)

    def test_select_time_field(self):
        valid = {
            ('10', '15'): time(10, 15),
            ('15', '15'): time(15, 15),
        }
        invalid = {
            ('42', '0'): ['Hour must be in 0..23'],
            ('1', '72'): ['Minute must be in 0..59'],
            ('', '15'): ['Hour must be in 0..23'],
            ('10', ''): ['Minute must be in 0..59'],
        }
        self.assertFieldOutput(SelectTimeField, valid, invalid,
                               empty_value=None)

    def test_select_time_field_advanced(self):
        "Ensure that 'empty' input behaves correctly when required is toggled."
        input = ('', '')

        # If the field is required, we expect a validation error to be raised.
        required = SelectTimeField(required=True)
        with self.assertRaises(ValidationError):
            required.clean(input)

        # If the field is optional, we expect None to be returned.
        optional = SelectTimeField(required=False)
        self.assertEqual(optional.clean(input), None)

    @override_settings(USE_TZ=False)
    def test_select_date_time_field(self):
        valid = {
            ('25', '9', '2013', '10', '15'): datetime(2013, 9, 25, 10, 15),
        }
        self.assertFieldOutput(SelectDateTimeField, valid, {},
                               empty_value=None)

    @override_settings(USE_TZ=True)
    def test_select_date_time_field_tz(self):
        est = pytz.timezone('Australia/Sydney')
        utc_dt = datetime(2013, 9, 25, 10, 15, tzinfo=timezone.utc)
        est_dt = datetime(2013, 9, 25, 10, 15, tzinfo=est)
        valid = {
            ('25', '9', '2013', '10', '15', 'UTC'): utc_dt,
            ('25', '9', '2013', '10', '15', 'Australia/Sydney'): est_dt,
        }
        invalid = {
            ('25', '9', '2013', '10', '15'): ['Please select a time zone.'],
            ('25', '9', '2013', '10', '15', ''): ['Please select a valid '
                                                  'time zone.'],
        }
        self.assertFieldOutput(SelectDateTimeField, valid, invalid,
                               empty_value=None)


class TestPermissionFormSet(TestCase):

    def setUp(self):
        self.UserModel = get_user_model()
        self.queryset = (
            Permission.objects
            .filter(content_type=ContentType.objects.get_for_model(
                SitemapNode
            ))
            .exclude(codename__startswith='add_')
        )
        self.instance = SitemapNode.objects.create(title='permissions')
        self.staff = factories.UserFactory.create(
            is_staff=True, is_superuser=True)
        self.regular = factories.UserFactory.create()

    def test_staff_only(self):
        formset_class = permissionformset_factory(
            SitemapNode, staff_only=True)
        formset = formset_class(queryset=self.queryset, instance=self.instance)
        self.assertQuerysetEqual(
            formset.forms[0].fields['users'].queryset,
            (repr(o) for o in self.UserModel.objects.filter(is_staff=True)),
        )

    def test_all_users(self):
        formset_class = permissionformset_factory(
            SitemapNode, staff_only=False)
        formset = formset_class(queryset=self.queryset, instance=self.instance)
        self.assertQuerysetEqual(
            formset.forms[0].fields['users'].queryset.order_by('pk'),
            (repr(o) for o in self.UserModel.objects.order_by('pk')),
        )

    def test_user_widget_checkbox_lte(self):
        "Less than equal to 5 users, should be iCheckboxSelectMultiple widget"
        formset_class = permissionformset_factory(
            SitemapNode, staff_only=False, max_checkboxes=5)
        formset = formset_class(queryset=self.queryset, instance=self.instance)
        self.assertEqual(
            formset.forms[0].fields['users'].queryset.count(),
            3,  # need to account for the AnonymousUser that guardian creates
        )

    def test_user_widget_checkbox_eq(self):
        "Equal to 3 users, should be iCheckboxSelectMultiple widget"
        formset_class = permissionformset_factory(
            SitemapNode, staff_only=False, max_checkboxes=3)
        formset = formset_class(queryset=self.queryset, instance=self.instance)
        self.assertEqual(
            formset.forms[0].fields['users'].queryset.count(),
            3,  # need to account for the AnonymousUser that guardian creates
        )

    def test_user_widget_select2(self):
        "More than 1 user, should not be iCheckboxSelectMultiple widget"
        formset_class = permissionformset_factory(
            SitemapNode, staff_only=False, max_checkboxes=1)
        formset = formset_class(queryset=self.queryset, instance=self.instance)
        self.assertEqual(
            formset.forms[0].fields['users'].queryset.count(),
            3,  # need to account for the AnonymousUser that guardian creates
        )



from __future__ import unicode_literals

import logging
import os

from django.conf import settings
from django.conf.urls import include, url
from django.contrib import messages
from django.core.files.storage import default_storage
from django.db.models import Q
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls import resolve, reverse
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from first import first
from touchtechnology.admin.base import AdminComponent
from touchtechnology.admin.mixins import AdminUrlMixin
from touchtechnology.admin.sites import site
from touchtechnology.common.decorators import require_POST_m, staff_login_required_m
from touchtechnology.common.models import SitemapNode
from touchtechnology.content.forms import (
    FileUploadForm, NewFolderForm, NewPlaceholderSitemapNodeForm, NewSitemapNodeForm, Page,
    PageContentFormset, PageForm, PlaceholderConfigurationBase, PlaceholderContentFormset,
    PlaceholderSitemapNodeForm, RedirectEditForm, SitemapNodeForm,
)
from touchtechnology.content.models import Chunk, PageContent, Placeholder, Redirect

logger = logging.getLogger(__name__)


class ContentAdminComponent(AdminComponent):
    verbose_name = _('Content')

    filter = Q(content_type__app_label='content') | \
        Q(content_type__isnull=True)

    def __init__(self, app, name="content", app_name="content"):
        super(ContentAdminComponent, self).__init__(app, name, app_name)

    def get_urls(self):
        folder_patterns = ([
            url(r'^add/$', self.edit_folder, name='add'),
            url(r'^(?P<pk>\d+)/$', self.edit_folder, name='edit'),
            url(r'^(?P<pk>\d+)/delete/$',
                self.delete_folder, name='delete'),
        ], self.app_name)

        page_patterns = ([
            url(r'^add/$',
                self.edit_page, name='add'),
            url(r'^(?P<pk>\d+)/$',
                self.edit_page, name='edit'),
            url(r'^(?P<pk>\d+)/delete/$',
                self.delete_page, name='delete'),
            url(r'^(?P<pk>\d+)/permission/$',
                self.perms_sitemapnode, name='perms'),
        ], self.app_name)

        redirect_patterns = ([
            url(r'^$', self.list_redirects, name='list'),
            url(r'^create/$', self.edit_redirect, name='add'),
            url(r'^(?P<pk>\d+)/$', self.edit_redirect, name='edit'),
            url(r'^(?P<pk>\d+)/delete/$',
                self.delete_redirect, name='delete'),
            url(r'^(?P<pk>\d+)/permission/$',
                self.perms_redirect, name='perms'),
        ], self.app_name)

        placeholder_patterns = ([
            url(r'^$', self.list_placeholders, name='list'),
            url(r'^create/$', self.edit_placeholder_x, name='add'),
            url(r'^(?P<pk>\d+)/$', self.edit_placeholder_x, name='edit'),
            url(r'^(?P<pk>\d+)/delete/$',
                self.delete_placeholder_x, name='delete'),
            # url(r'^(?P<pk>\d+)/permission/$',
            #     self.perms_placeholder, name='delete'),
        ], self.app_name)

        application_patterns = ([
            url(r'^add/$', self.edit_application_node, name='add'),
            url(r'^(?P<pk>\d+)/$',
                self.edit_application_node, name='edit'),
            url(r'^(?P<pk>\d+)/delete/$',
                self.delete_application_node, name='delete'),
            url(r'^(?P<pk>\d+)/permission/$',
                self.perms_sitemapnode, name='perms'),
        ], self.app_name)

        chunk_patterns = ([
            url(r'^$', self.list_chunks, name='list'),
            url(r'^create/$', self.edit_chunk, name='add'),
            url(r'^(?P<pk>\d+)/$', self.edit_chunk, name='edit'),
            url(r'^(?P<pk>\d+)/delete/$',
                self.delete_chunk, name='delete'),
            url(r'^(?P<pk>\d+)/permission/$',
                self.perms_chunk, name='perms'),
        ], self.app_name)

        files_patterns = ([
            url(r'^(?P<path>.+/)?(?P<filename>[^/]+?)/rm/$',
                self.rm, name='delete'),
            url(r'^(?P<path>.+/)rmdir/$', self.rm, name='delete'),
            url(r'^$', self.ls, name='list'),
            url(r'^(?P<path>.+)$', self.ls, name='edit'),
        ], self.app_name)

        urlpatterns = [
            url(r'^$', self.list_pages, name='index'),
            url(r'^folder/', include(folder_patterns, namespace='folder')),
            url(r'^page/', include(page_patterns, namespace='page')),
            url(r'^redirect/', include(redirect_patterns, namespace='redirect')),
            url(r'^placeholder/', include(placeholder_patterns, namespace='real_placeholder')),
            url(r'^application/', include(application_patterns, namespace='placeholder')),
            url(r'^chunk/', include(chunk_patterns, namespace='chunk')),
            url(r'^files/', include(files_patterns, namespace='media')),
        ]
        return urlpatterns

    def dropdowns(self):
        dl = (
            (_("Sitemap"), self.reverse('index'), 'sitemap'),
            (_("Media"), self.reverse('media:list'), 'folder-open'),
            (_("Chunks"), self.reverse('chunk:list'), 'pencil-square-o'),
            (_("Redirects"), self.reverse('redirect:list'), 'reply'),
            (_("Applications"),
             self.reverse('real_placeholder:list'), 'puzzle-piece'),
        )
        return dl

    # Generic views - trial before moving onto Application

    def generic_parent_edit_related(self, request, pk,
                                    parent_model,
                                    child_model,
                                    child_related_model,
                                    parent_form_class=None,
                                    child_form_class=None,
                                    child_formset_class=None,
                                    post_save_redirect=None,
                                    *args, **kwargs):
        if pk is not None:
            instance = get_object_or_404(parent_model, pk=pk)
        else:
            instance = None
        if post_save_redirect is None:
            post_save_redirect = self.redirect(self.reverse('index'))
        return self.generic_edit_related(
            request, child_model, child_related_model,
            instance=instance,
            form_class=child_form_class,
            form_kwargs={
                'parent_form_class': parent_form_class,
                'parent_form_kwargs': {'user': request.user},
            },
            formset_class=child_formset_class,
            post_save_redirect=post_save_redirect,
            extra_context=kwargs)

    # Shared perms view

    def perms_sitemapnode(self, request, pk, **extra_context):
        return self.generic_permissions(request, SitemapNode, pk=pk,
                                        **extra_context)

    # Folder views -- folders are just SitemapNode's with no links

    @staff_login_required_m
    def edit_folder(self, request, pk=None, *args, **kwargs):
        form_class = NewSitemapNodeForm if pk is None else SitemapNodeForm
        return self.generic_edit(request, SitemapNode, pk=pk,
                                 form_class=form_class,
                                 related=(),
                                 permission_required=True,
                                 extra_context=kwargs)

    @staff_login_required_m
    def delete_folder(self, request, pk, *args, **kwargs):
        node = get_object_or_404(SitemapNode, pk=pk)
        if not node.is_leaf_node():
            raise Http404
        return self.generic_delete(
            request, SitemapNode, pk=pk, extra_context=kwargs)

    # Page views

    @staff_login_required_m
    def list_pages(self, request, **extra_context):
        try:
            root = SitemapNode.objects.get(pk=request.GET['r'])
            descendants = root.get_descendants()
            nodes = descendants.filter(self.filter) \
                               .filter(level=root.level + 1)
        except KeyError:
            nodes = SitemapNode.objects.filter(self.filter) \
                                       .filter(level=0)
        return self.generic_list(request, nodes,
                                 paginate_by=0,
                                 permission_required=True,
                                 extra_context=extra_context)

    @staff_login_required_m
    def edit_page(self, request, pk=None, **extra_context):
        post_save_redirect = None
        if pk is not None:
            instance = get_object_or_404(SitemapNode, pk=pk)
            if instance.parent_id:
                post_save_redirect = self.redirect(
                    self.reverse('index') + "?r=%d" % instance.parent_id)
        return self.generic_parent_edit_related(
            request, pk, parent_model=SitemapNode, child_model=Page,
            child_related_model=PageContent,
            parent_form_class=SitemapNodeForm, child_form_class=PageForm,
            child_formset_class=PageContentFormset,
            post_save_redirect=post_save_redirect,
            **extra_context)

    @staff_login_required_m
    def delete_page(self, request, pk, *args, **kwargs):
        node = get_object_or_404(SitemapNode, pk=pk)

        if request.method == 'POST':
            # verify that we are only operating on a leaf node
            if not node.is_leaf_node():
                raise Http404
            # if this was not a POST request, we don't make changes to the
            # database, we just display the warning page.
            page = node.object
            page.content.all().delete()
            page.delete()
            node.delete()
            redirect_to = self.reverse('index')
            if node.parent:
                redirect_to += '?r=%d' % node.parent_id
            return HttpResponseRedirect(redirect_to)

        context = {
            'object': node,
        }
        context.update(kwargs)

        templates = self.template_path('delete_confirm.html')
        return self.render(request, templates, context)

    # Redirect views

    @staff_login_required_m
    def list_redirects(self, request, *args, **kwargs):
        return self.generic_list(request, Redirect,
                                 permission_required=True,
                                 extra_context=kwargs)

    @staff_login_required_m
    def edit_redirect(self, request, pk=None, *args, **kwargs):
        return self.generic_edit(request, Redirect, pk=pk,
                                 form_class=RedirectEditForm,
                                 permission_required=True,
                                 extra_context=kwargs)

    @staff_login_required_m
    def delete_redirect(self, request, pk, *args, **kwargs):
        return self.generic_delete(request, Redirect, pk=pk,
                                   permission_required=True,
                                   extra_context=kwargs)

    @staff_login_required_m
    def perms_redirect(self, request, pk, *args, **kwargs):
        return self.generic_permissions(request, Redirect, pk=pk,
                                        **kwargs)

    # Application views

    @staff_login_required_m
    def list_placeholders(self, request, *args, **kwargs):
        return self.generic_list(request, Placeholder,
                                 paginate_by=0,
                                 permission_required=True,
                                 extra_context=kwargs)

    @staff_login_required_m
    def edit_placeholder_x(self, request, pk=None, *args, **kwargs):
        return self.generic_edit(request, Placeholder, pk=pk,
                                 permission_required=True,
                                 extra_context=kwargs)

    @staff_login_required_m
    def delete_placeholder_x(self, request, pk, *args, **kwargs):
        return self.generic_delete(request, Placeholder, pk=pk,
                                   permission_required=True,
                                   extra_context=kwargs)

    @staff_login_required_m
    def edit_application_node(self, request, pk=None, **extra_context):
        kwargs_form_class = PlaceholderConfigurationBase

        if pk is None:
            instance = SitemapNode()
            form_class = NewPlaceholderSitemapNodeForm
        else:
            instance = get_object_or_404(SitemapNode, pk=pk)
            form_class = PlaceholderSitemapNodeForm
            try:
                site = instance.object.site(instance)
            except ValueError:
                logger.exception("Application is unavailable, "
                                 "no custom form class.")
            else:
                kwargs_form_class = site.kwargs_form_class or kwargs_form_class

        if request.method == 'POST':
            form = form_class(data=request.POST, instance=instance,
                              user=request.user)
            kw = kwargs_form_class(data=request.POST, instance=instance,
                                   prefix='kw', user=request.user)
            copy = PlaceholderContentFormset(
                data=request.POST, instance=instance, prefix='copy')
            if form.is_valid() and kw.is_valid() and copy.is_valid():
                form.save()
                kw.save()
                copy.save()

                redirect_to = self.reverse('index')
                if instance.parent:
                    redirect_to += '?r=%d' % instance.parent_id

                # post_save_callback if we ever move this to a generic_edit
                # pattern in future.
                instance.object.invalidate_cache()

                return HttpResponseRedirect(redirect_to)
        else:
            form = form_class(instance=instance, user=request.user)
            kw = kwargs_form_class(
                instance=instance, prefix='kw', user=request.user)
            copy = PlaceholderContentFormset(instance=instance, prefix='copy')

        context = {
            'form': form,
            'formsets': {
                'kwargs': kw,
                'copy': copy,
            },
            'formset_media': kw.media + copy.media,
            'node': instance,
            'model': Placeholder.objects.none(),
        }
        context.update(extra_context)

        templates = self.template_path('edit_application.html')
        return self.render(request, templates, context)

    @require_POST_m
    @staff_login_required_m
    def delete_application_node(self, request, pk, *args, **kwargs):
        node = get_object_or_404(SitemapNode, pk=pk)

        if request.method == 'POST':
            # verify that we are only operating on a leaf node
            if not node.is_leaf_node():
                raise Http404
            redirect_to = reverse('admin:content:index', current_app=self.app)
            if node.parent:
                redirect_to += '?r=%d' % node.parent_id
            # if this was not a POST request, we don't make changes to the
            # database, we just display the warning page.
            node.delete()

            return HttpResponseRedirect(redirect_to)

        context = {
            'object': node,
        }
        context.update(kwargs)

        templates = self.template_path('delete_confirm.html')
        return self.render(request, templates, context)

    # Chunk views

    @staff_login_required_m
    def list_chunks(self, request, *args, **kwargs):
        return self.generic_list(request, Chunk,
                                 extra_context=kwargs)

    @staff_login_required_m
    def edit_chunk(self, request, pk=None, *args, **kwargs):
        return self.generic_edit(request, Chunk, pk=pk,
                                 extra_context=kwargs)

    @staff_login_required_m
    def delete_chunk(self, request, pk, **extra_context):
        return self.generic_delete(request, Chunk, pk=pk,
                                   extra_context=extra_context)

    @staff_login_required_m
    def perms_chunk(self, request, pk, *args, **kwargs):
        return self.generic_permissions(request, Chunk, pk=pk,
                                        **kwargs)

    @staff_login_required_m
    def ls(self, request, path=None, *args, **kwargs):
        try:
            directories, files = default_storage.listdir(path or ".")
        except OSError as e:
            raise Http404(e)

        directories.sort()
        files.sort()

        parentdir = os.path.dirname(path[:-1]) or 'ROOT' if path else ''

        writable = True  # presume it is, and fail gracefully if it isn't
        if path and hasattr(default_storage, 'writable'):
            writable = default_storage.writable(path)

        # Both types for Form expect a path to be passed in, set up the base.
        form_kwargs = dict(path=path)

        if request.method == 'POST':
            # Give the Forms the POST and FILES from the request
            form_kwargs.update(dict(data=request.POST, files=request.FILES))

            folder_form = NewFolderForm(prefix='folder', **form_kwargs)
            file_form = FileUploadForm(prefix='file', **form_kwargs)

            if folder_form.is_valid() and file_form.is_valid():
                redirect_to = '.'
                saved = None

                if folder_form.has_changed():
                    saved = folder_form.save()
                    logger.debug('created folder %s', saved)
                    folder_name = os.path.basename(saved)
                    messages.success(
                        request, _('Opened new folder "%s".') % folder_name)
                    redirect_to = folder_name + '/'  # FIXME reverse this

                if file_form.has_changed():
                    saved = file_form.save()
                    file_name = os.path.basename(saved)
                    messages.success(
                        request, _('Uploaded new file "%s"') % file_name)

                if saved is None:
                    messages.warning(
                        request, _('There were no changes to be saved.'))

                return self.redirect(redirect_to)

        else:
            folder_form = NewFolderForm(prefix='folder', **form_kwargs)
            file_form = FileUploadForm(prefix='file', **form_kwargs)

        def _directories(d):
            fullpath = os.path.join(path or '', d, '')
            return fullpath, d, default_storage.listdir(fullpath)

        directories = [_directories(d) for d in directories]
        match = resolve(request.path)

        @python_2_unicode_compatible
        class FileOrFolder(AdminUrlMixin, object):
            def __init__(s, pk, name):
                s.pk = pk
                s.name = name

            def __str__(s):
                return s.name

            def __repr__(s):
                return '<%s: %s, %r>' % (s.__class__.__name__, s.name, s.urls)

            def _get_admin_namespace(s):
                return match.namespace

            def _get_url_args(s):
                return (s.pk,)

        class File(FileOrFolder):
            icon = 'file'

            @cached_property
            def urls(s):
                namespace = s._get_admin_namespace()
                args = s._get_url_args()
                crud = {
                    'detail': os.path.join(settings.MEDIA_URL, *args),
                    'delete': reverse(
                        '%s:delete' % namespace, args=args),
                }
                return crud

            def _get_url_args(s):
                return filter(None, (path, s.pk))

        class Folder(FileOrFolder):
            icon = 'folder'

        object_list = (
            [Folder(d[0], d[1]) for d in directories] +
            [File(f, f) for f in files]
        )
        from collections import namedtuple
        context = {
            'base_url': default_storage.base_url,
            'path': path,
            'parent': parentdir,
            'directories': directories,
            'files': files,
            'folder_form': folder_form,
            'file_form': file_form,
            'writable': writable,
            'object_list': object_list,
            'queryset': Page.objects.all(),
            'list_template': first(
                self.template_path('list.inc.html', 'file')),
            'model': namedtuple('file', ()),
        }
        context.update(kwargs)

        templates = self.template_path('list.html', 'file')
        return self.render(request, templates, context)

    @staff_login_required_m
    def rm(self, request, path=None, filename=None, *args, **kwargs):
        try:
            directories, files = default_storage.listdir(path or '.')
        except OSError as e:
            logger.exception('path: %r', path)
            raise Http404(e[1])

        @python_2_unicode_compatible
        class File(object):
            def __init__(self, path):
                self.path = path

            def __str__(self):
                return self.path

            def is_leaf_node(self):
                return True

        class Folder(File):
            def __init__(self, path):
                self.path = os.path.dirname(path)

            def is_leaf_node(self):
                listdir = default_storage.listdir(self.path)
                logger.debug(listdir)
                return not any(filter(None, listdir))

        fullpath = os.path.join(path or '', filename or '')

        if filename:
            cls, action, tpl = File, default_storage.delete, 'file'
            item = os.path.basename(fullpath)
        elif hasattr(default_storage, 'rmdir'):
            cls, action, tpl = Folder, default_storage.rmdir, 'folder'
            item = os.path.basename(os.path.dirname(fullpath))
        else:
            raise Http404("We can't handle this, as the "
                          "backend storage system does not "
                          "support removing directories.")

        if path is not None:
            parentdir = os.path.normpath(os.path.join(path, '..')) + '/'
        else:
            parentdir = ''

        obj = cls(fullpath)
        if not obj.is_leaf_node():
            messages.error(request, _('The folder "%s" is not empty.') % item)
            return self.redirect(
                self.reverse('media:edit', kwargs={'path': parentdir}))

        if request.method == 'POST':
            action(fullpath)
            messages.success(
                request,
                'Successfully deleted %s "%s"' % (tpl, item))
            if path:
                if cls is Folder:
                    return self.redirect(
                        self.reverse('media:edit', kwargs={'path': parentdir}))
                return self.redirect(
                    self.reverse('media:edit', kwargs={'path': path}))
            return self.redirect(self.reverse('media:list'))

        context = {
            'path': path,
            'parentdir': parentdir,
            'filename': filename,
            'object': obj,
        }
        context.update(kwargs)

        templates = self.template_path('delete_confirm.html', tpl)
        return self.render(request, templates, context)


site.register(ContentAdminComponent)



from __future__ import unicode_literals


class NodeRelationMixin(object):
    """
    A mixin to allow a Node subclass to determine the type of
    relationship it has with another. Used by the ``navigation``
    template tag.
    """
    def rel(self, other):
        """
        Returns the relationship as an uppercase string. Allowed values
        are as follows:

         * ME
         * ROOT
         * PARENT
         * ANCESTOR
         * SIBLING (share the same parent)
         * UNCLE (sibling of parent)
         * DESCENDANT

        For nodes that do not have a direct relationship, the value
        returned is generic dependant upon the type of instance - used
        only for debugging purposes.

         * SITEMAP
         * FAUX
        """
        from .models import SitemapNode
        from .utils import FauxNode

        if other == self:
            return 'ME'

        if self.parent and other == self.parent:
            return 'PARENT'

        if other.parent == self.parent:
            return 'SIBLING'

        if other.parent and other.parent == self:
            return 'DESCENDANT'

        if self.parent:
            prel = self.parent.rel(other)

            if prel == 'SIBLING':
                return 'UNCLE'

            if prel == 'PARENT':
                return 'ANCESTOR'

            if prel == 'ANCESTOR':
                return prel

        if other.parent is None:
            return 'ROOT'

        if other.tree_id == self.tree_id:
            if other.level < self.level:
                return 'ANCESTOR'

        if isinstance(other, SitemapNode):
            return 'SITEMAP'
        if isinstance(other, FauxNode):
            return 'FAUX'

        raise ValueError('You must provide a "SitemapNode" or '
                         '"FauxNode" for comparison.')



import uuid

from django.conf import settings
from django.conf.urls import include, url
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.forms.models import modelform_factory
from django.utils.translation import ugettext_lazy as _
from touchtechnology.admin.base import AdminComponent
from touchtechnology.admin.forms import GroupEditForm, UserEditForm
from touchtechnology.common.decorators import staff_login_required_m

ANONYMOUS_USER_ID = getattr(settings, 'ANONYMOUS_USER_ID', None)


class UsersGroups(AdminComponent):
    verbose_name = _('Users & Groups')

    user_order_by = ('last_name', 'first_name')

    group_class = Group
    group_order_by = ('name',)

    unprotected = False

    def __init__(self, app, name="auth", app_name="auth"):
        super(UsersGroups, self).__init__(app, name, app_name)
        self.user_class = get_user_model()

    @property
    def template_base(self):
        return 'touchtechnology/admin'

    def get_urls(self):
        user_patterns = ([
            url(r'^$', self.list_users, name='list'),
            url(r'^add/$', self.edit_user, name='add'),
            url(r'^(?P<pk>\d+)/$', self.edit_user, name='edit'),
            url(r'^(?P<pk>\d+)/delete/$', self.delete_user, name='delete'),
            url(r'^(?P<pk>\d+)/permission/$',
                self.perms_user, name='perms'),
        ], self.app_name)

        group_patterns = ([
            url(r'^$', self.list_groups, name='list'),
            url(r'^add/$', self.edit_group, name='add'),
            url(r'^(?P<pk>\d+)/$', self.edit_group, name='edit'),
            url(r'^(?P<pk>\d+)/delete/$',
                self.delete_group, name='delete'),
            url(r'^(?P<pk>\d+)/permission/$',
                self.perms_group, name='perms'),
        ], self.app_name)

        urlpatterns = [
            url(r'^$', self.index, name='index'),
            url(r'^user/', include(user_patterns, namespace='users')),
            url(r'^group/', include(group_patterns, namespace='groups')),
        ]
        return urlpatterns

    def dropdowns(self):
        dl = (
            (_('Users'), self.reverse('users:list'), 'user'),
            (_('Groups'), self.reverse('groups:list'), 'users'),
        )
        return dl

    @staff_login_required_m
    def index(self, request, *args, **kwargs):
        return self.redirect(self.reverse('users:list'))

    @staff_login_required_m
    def list_users(self, request, **extra_context):
        queryset = self.user_class.objects \
                                  .exclude(pk=ANONYMOUS_USER_ID) \
                                  .order_by(*self.user_order_by)
        return self.generic_list(request, queryset,
                                 paginate_by=25,
                                 permission_required=True,
                                 search=('first_name', 'last_name', 'email'),
                                 extra_context=extra_context)

    @staff_login_required_m
    def edit_user(self, request, pk=None, **extra_context):
        # when performing a creation we need to pass an instance with a random
        # password already set so that the user can initiate a password reset.
        instance = None

        if pk is None:
            instance = self.user_class()
            # generate a password that we will never retain in the clear and
            # would be very hard to brute force attack due to it's length.
            password = uuid.uuid4().hex

            # set the random password so that it's not possible to sign-in to
            # this account without first performing a password reset cycle.
            instance.set_password(password)

        # create a ModelForm using our base UserEditForm, but overload the
        # Meta.fields with value from custom UserModel.USER_EDIT_FORM_FIELDS
        # if it is present. Always exclude certain fields.
        fields = getattr(self.user_class, 'USER_EDIT_FORM_FIELDS', '__all__')
        exclude = ('password', 'last_login', 'date_joined')
        form_class = modelform_factory(self.user_class, form=UserEditForm,
                                       fields=fields, exclude=exclude)

        return self.generic_edit(request, self.user_class,
                                 pk=pk, instance=instance,
                                 form_class=form_class,
                                 related=(),
                                 permission_required=True,
                                 post_save_redirect=self.redirect(
                                     self.reverse('users:list')),
                                 extra_context=extra_context)

    @staff_login_required_m
    def delete_user(self, request, pk, **extra_context):
        return self.generic_delete(request, self.user_class, pk=pk,
                                   permission_required=True)

    @staff_login_required_m
    def perms_user(self, request, pk, **extra_context):
        return self.generic_permissions(request, self.user_class, pk=pk,
                                        **extra_context)

    @staff_login_required_m
    def list_groups(self, request, **extra_context):
        queryset = self.group_class.objects.order_by(*self.group_order_by)
        return self.generic_list(request, queryset,
                                 paginate_by=25,
                                 permission_required=True,
                                 extra_context=extra_context)

    @staff_login_required_m
    def edit_group(self, request, pk=None, **extra_context):
        return self.generic_edit(request, self.group_class, pk=pk,
                                 form_class=GroupEditForm,
                                 related=(),
                                 permission_required=True,
                                 post_save_redirect=self.redirect(
                                     self.reverse('groups:list')),
                                 extra_context=extra_context)

    @staff_login_required_m
    def delete_group(self, request, pk, **extra_context):
        return self.generic_delete(request, self.group_class, pk=pk,
                                   permission_required=True)

    @staff_login_required_m
    def perms_group(self, request, pk, **extra_context):
        return self.generic_permissions(request, self.group_class, pk=pk,
                                        **extra_context)



import six
from test_plus import TestCase
from tournamentcontrol.competition.models import LadderEntry, LadderSummary
from tournamentcontrol.competition.tests import factories


class SignalHandlerTests(TestCase):

    def test_ladder_entry_match_no_score(self):
        factories.MatchFactory.create()
        self.assertQuerysetEqual(
            LadderEntry.objects.all(),
            LadderEntry.objects.none())

    def test_ladder_entry_match_with_score(self):
        factories.MatchFactory.create(home_team_score=5, away_team_score=2)
        six.assertCountEqual(
            self,
            [
                (1, 0, 0, 5, 2, 3, 3),
                (0, 1, 0, 2, 5, -3, 3),
            ],
            LadderEntry.objects.values_list(
                'win', 'loss', 'draw', 'score_for', 'score_against',
                'diff', 'margin'))

    def test_ladder_summary_3_match_series(self):
        # seed a match which will produce all required relations
        match = factories.MatchFactory.create(home_team_score=5, away_team_score=2)

        # build two more matches with known results to produce LadderSummary
        factories.MatchFactory.create(
            home_team=match.home_team, home_team_score=3,
            away_team=match.away_team, away_team_score=4,
            stage=match.stage)
        factories.MatchFactory.create(
            home_team=match.home_team, home_team_score=2,
            away_team=match.away_team, away_team_score=2,
            stage=match.stage)

        six.assertCountEqual(
            self,
            [
                (3, 1, 1, 1, 10, 8, 2),
                (3, 1, 1, 1, 8, 10, -2),
            ],
            LadderSummary.objects.values_list(
                'played', 'win', 'loss', 'draw',
                'score_for', 'score_against', 'difference'))



import textwrap

from django.test import TestCase
from tournamentcontrol.competition import utils


class UtilTests(TestCase):

    def test_round_robin_even(self):
        self.assertEqual(
            [
                [(1, 4), (2, 3)],
                [(1, 3), (4, 2)],
                [(1, 2), (3, 4)],
            ],
            utils.round_robin([1, 2, 3, 4]))

    def test_round_robin_odd(self):
        self.assertEqual(
            [
                [(1, 0), (2, 3)],
                [(1, 3), (0, 2)],
                [(1, 2), (3, 0)],
            ],
            utils.round_robin([1, 2, 3]))

    def test_round_robin_format_even(self):
        self.assertEqual(
            textwrap.dedent("""
                ROUND
                1: 1 vs 4
                2: 2 vs 3
                ROUND
                3: 1 vs 3
                4: 4 vs 2
                ROUND
                5: 1 vs 2
                6: 3 vs 4
                """).strip(),
            utils.round_robin_format([1, 2, 3, 4]))

    def test_sum_dict_integer(self):
        d1 = utils.SumDict({'a': 1, 'b': 2})
        d2 = utils.SumDict({'b': 1, 'c': 2})
        self.assertEqual(
            d1 + d2,
            utils.SumDict({'a': 1, 'b': 3, 'c': 2}))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django.db.models.deletion
import touchtechnology.common.db.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('competition', '0017_ranking_feature'),
    ]
    operations = [
        migrations.AlterModelOptions(
            name='clubassociation',
            options={
                'ordering': ('person__last_name', 'person__first_name'),
                'verbose_name': 'Official',
                'verbose_name_plural': 'Officials',
            },
        ),
        migrations.AlterModelOptions(
            name='competition',
            options={},
        ),
        migrations.AlterModelOptions(
            name='division',
            options={},
        ),
        migrations.AlterModelOptions(
            name='divisionexclusiondate',
            options={'verbose_name': 'exclusion date'},
        ),
        migrations.AlterModelOptions(
            name='drawformat',
            options={'ordering': ('teams', 'name')},
        ),
        migrations.AlterModelOptions(
            name='match',
            options={
                'get_latest_by': 'datetime',
                'ordering': (
                    'date',
                    'stage',
                    'round',
                    'is_bye',
                    'time',
                    'play_at__ground__order',
                    'pk',
                ),
                'verbose_name_plural': 'matches',
            },
        ),
        migrations.AlterModelOptions(
            name='person',
            options={
                'ordering': ('last_name', 'first_name'),
                'verbose_name_plural': 'people',
            },
        ),
        migrations.AlterModelOptions(
            name='season',
            options={},
        ),
        migrations.AlterModelOptions(
            name='seasonexclusiondate',
            options={
                'ordering': ('date',),
                'verbose_name': 'exclusion date',
            },
        ),
        migrations.AlterModelOptions(
            name='seasonmatchtime',
            options={
                'verbose_name': 'time slot',
            },
        ),
        migrations.AlterModelOptions(
            name='simplescorematchstatistic',
            options={
                'get_latest_by': 'match',
                'ordering': ('match', 'number'),
            },
        ),
        migrations.AlterModelOptions(
            name='stagegroup',
            options={
                'ordering': ('order',),
                'verbose_name': 'pool',
            },
        ),
        migrations.AlterModelOptions(
            name='team',
            options={
                'ordering': (
                    '-division__season__start_date',
                    'division__order',
                    'stage_group__order',
                    'order',
                ),
                'permissions': (
                    ('view_team', 'Can view team'),
                ),
            },
        ),
        migrations.AlterModelOptions(
            name='teamassociation',
            options={
                'ordering': (
                    '-is_player',
                    'number',
                    'person__last_name',
                    'person__first_name',
                ),
                'verbose_name': 'linked person',
                'verbose_name_plural': 'linked people',
            },
        ),
        migrations.AlterModelOptions(
            name='undecidedteam',
            options={
                'ordering': ('stage_group', 'formula')
            },
        ),
        migrations.AlterField(
            model_name='club',
            name='primary',
            field=touchtechnology.common.db.models.ForeignKey(
                to='competition.Person',
                verbose_name='Primary contact',
                related_name='+',
                null=True,
                blank=True,
                help_text='Appears on the front-end with other club '
                          'information.',
            ),
        ),
        migrations.AlterField(
            model_name='division',
            name='draft',
            field=models.BooleanField(
                default=False,
                help_text='Marking a division as draft will prevent matches '
                          'from being visible in the front-end.',
            ),
        ),
        migrations.AlterField(
            model_name='division',
            name='games_per_day',
            field=models.SmallIntegerField(
                null=True,
                help_text='In Tournament mode, specify how many matches per '
                          'day should be scheduled by the automatic draw '
                          'generator.',
            ),
        ),
        migrations.AlterField(
            model_name='division',
            name='include_forfeits_in_played',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='match',
            name='away_team_eval_related',
            field=touchtechnology.common.db.models.ForeignKey(
                to='competition.Match',
                related_name='+',
                null=True,
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name='match',
            name='forfeit_winner',
            field=touchtechnology.common.db.models.ForeignKey(
                to='competition.Team',
                related_name='+',
                null=True,
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name='match',
            name='home_team_eval_related',
            field=touchtechnology.common.db.models.ForeignKey(
                to='competition.Match',
                related_name='+',
                null=True,
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name='person',
            name='email',
            field=models.EmailField(max_length=254, blank=True),
        ),
        migrations.AlterField(
            model_name='person',
            name='first_name',
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name='person',
            name='last_name',
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name='season',
            name='mvp_results_public',
            field=touchtechnology.common.db.models.DateTimeField(
                verbose_name='MVP public at',
                null=True,
                blank=True,
                help_text='The time when the results of the MVP voting will '
                          'be made public on the website. Leave blank to show '
                          'at all times.',
            ),
        ),
        migrations.AlterField(
            model_name='simplescorematchstatistic',
            name='mvp',
            field=models.SmallIntegerField(
                verbose_name='MVP',
                null=True,
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name='simplescorematchstatistic',
            name='points',
            field=models.SmallIntegerField(
                verbose_name='Points',
                null=True,
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name='stage',
            name='carry_ladder',
            field=models.BooleanField(
                verbose_name=b'Carry over points',
                default=False,
                help_text='Set this to <b>Yes</b> if this stage should carry '
                          'over values from the previous stage.',
            ),
        ),
        migrations.AlterField(
            model_name='stage',
            name='follows',
            field=touchtechnology.common.db.models.ForeignKey(
                to='competition.Stage',
                related_name='preceeds',
                on_delete=django.db.models.deletion.SET_NULL,
                null=True,
                blank=True,
                help_text='When progressing teams into this stage, which '
                          'earlier stage should be used for determining '
                          'positions.<br />Default is the immediately '
                          'preceeding stage.',
            ),
        ),
        migrations.AlterField(
            model_name='stage',
            name='keep_ladder',
            field=models.BooleanField(
                verbose_name=b'Keep a ladder',
                default=True,
                help_text='Set this to <b>No</b> if this stage does not need '
                          'to keep a competition ladder.<br />Usually set to '
                          'No for a Final Series or a Knockout stage.',
            ),
        ),
        migrations.AlterField(
            model_name='stage',
            name='keep_mvp',
            field=models.BooleanField(
                verbose_name=b'Keep MVP stats',
                default=True,
                help_text='Set this to <b>No</b> if this stage does not need '
                          'to keep track of MVP points.<br />Usually set to '
                          'No for a Final Series.',
            ),
        ),
        migrations.AlterField(
            model_name='stage',
            name='scale_group_points',
            field=models.BooleanField(
                default=False,
                help_text='In stages with multiple pools, adjust points in '
                          'the smaller groups to compensate for the reduced '
                          'opportunity to score points.<br />'
                          'You <strong>should</strong> also set 0 points for '
                          'Bye matches.',
            ),
        ),
        migrations.AlterField(
            model_name='stagegroup',
            name='carry_ladder',
            field=models.BooleanField(
                verbose_name=b'Carry over points',
                default=False,
                help_text='Set this to <b>Yes</b> if the ladder for this pool '
                          'should carry over values from the previous '
                          'stage.<br />'
                          'Will only apply for matches played against teams '
                          'that are now in this group.',
            ),
        ),
        migrations.AlterField(
            model_name='team',
            name='names_locked',
            field=models.BooleanField(
                default=False,
                help_text='When the team name is locked, the team manager '
                          'will not be able to change their team name.<br />'
                          'As a tournament manager you can always change the '
                          'names.',
            ),
        ),
    ]



import logging
from functools import wraps

logger = logging.getLogger(__name__)


def disable_for_loaddata(signal_handler):
    """
    Decorator that turns off signal handlers when loading
    fixture data.
    """
    @wraps(signal_handler)
    def wrapper(*args, **kwargs):
        raw = kwargs.get('raw', False)
        if raw:
            instance = kwargs.get('instance')
            logger.debug('Skipping signal handler %r for instance %r.',
                         signal_handler, instance)
            return
        signal_handler(*args, **kwargs)
    return wrapper



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


def prefix_with_hash(apps, schema_editor):
    Season = apps.get_model("competition", "Season")
    for season in Season.objects.exclude(hashtag__isnull=True):
        season.hashtag = '#' + season.hashtag
        season.save()


def remove_hash_prefix(apps, schema_editor):
    Season = apps.get_model("competition", "Season")
    for season in Season.objects.filter(hashtag__startswith="#"):
        season.hashtag = season.hashtag[1:]
        season.save()


class Migration(migrations.Migration):

    dependencies = [
        ('competition', '0011_custom_twitter_ladderpoints_fields'),
    ]

    operations = [
        migrations.AlterField(
            model_name='season',
            name='hashtag',
            field=models.CharField(validators=[django.core.validators.RegexValidator(b'^(?:#)(\\w+)$', 'Enter a valid value. Make sure you include the # symbol.')], max_length=30, blank=True, help_text='Your official <em>hash tag</em> for social media promotions.', null=True, verbose_name=b'Hash Tag'),
        ),
        migrations.RunPython(prefix_with_hash, remove_hash_prefix)
    ]



import base64
import copy
import os.path

from django.test.utils import override_settings
from test_plus import TestCase
from touchtechnology.common.tests.factories import UserFactory
from touchtechnology.news.models import Article

try:
    from io import BytesIO
except ImportError:
    from StringIO import StringIO as BytesIO


TEST_DIR = os.path.join(os.path.dirname(__file__), 'html')


class ImageFieldTestCase(TestCase):

    def setUp(self):
        self.superuser = UserFactory.create(is_staff=True, is_superuser=True)

        # Single transparent pixel PNG image, base64 encoded.
        png_data = base64.b64decode(
            "iVBORw0KGgoAAAANSUhEUgAAAAEAAAABAQAAAAA3bvkkAAAACklEQVQImWNoAAAA"
            "ggCByxOyYQAAAABJRU5ErkJggg==")

        self.data_no_image = {
            'form-headline': 'Headline',
            'form-image': None,
            'form-abstract': 'This is the abstract',
            'form-published': '2017-01-01 05:30:00',
            'formset-0-copy': '',
            'formset-0-label': 'copy',
            'formset-0-sequence': '1',
            'formset-TOTAL_FORMS': '1',
            'formset-INITIAL_FORMS': '0',
            'formset-MIN_NUM_FORMS': '0',
            'formset-MAX_NUM_FORMS': '100',
        }
        self.data_with_image = copy.copy(self.data_no_image)
        self.data_with_image['form-image'] = BytesIO(png_data)
        # From Django 1.11 the image field validates the filename.
        self.data_with_image['form-image'].name = 'pixel.png'

    def test_image_required_no_image(self):
        with self.login(self.superuser):
            self.post('admin:news:article:add', data=self.data_no_image)
        self.assertFormError(
            self.last_response, 'form', 'image', 'This field is required.')
        self.assertEqual(0, Article.objects.count())

    def test_image_required_with_image(self):
        with self.login(self.superuser):
            self.post('admin:news:article:add', data=self.data_with_image)
        self.response_302()
        self.assertEqual(1, Article.objects.count())

    @override_settings(TOUCHTECHNOLOGY_NEWS_IMAGE_REQUIRED=False)
    def test_image_not_required_no_image(self):
        with self.login(self.superuser):
            self.post('admin:news:article:add', data=self.data_no_image)
        self.response_302()
        self.assertEqual(1, Article.objects.count())

    @override_settings(TOUCHTECHNOLOGY_NEWS_IMAGE_REQUIRED=False)
    def test_image_not_required_with_image(self):
        with self.login(self.superuser):
            self.post('admin:news:article:add', data=self.data_with_image)
        self.response_302()
        self.assertEqual(1, Article.objects.count())
